import sys
import syslog
import time
import os
import shutil
import subprocess as sub
from  PyQt5 import QtCore,QtGui,uic,QtWidgets
from  LI_Generic  import ListItemGeneric
from  WebDialog import Dialog
from  HelpDialog import HDialog
from  ripper import Ripper
from  Strings import Strings

# ui file 
# this will not be used rather compiled...   

# uic_file = "ripper.ui"
# Ui_MainWindow,QtBaseClass = uic.loadUiType(uic_file)


from ui import Ui_MainWindow

class MyApp(QtWidgets.QMainWindow, Ui_MainWindow):

    def __init__(self):

        
        QtWidgets.QMainWindow.__init__(self)
        Ui_MainWindow.__init__(self)
        self.setupUi(self)

  #setup buttons and actions
        self.addBtn.clicked.connect(self.addEntry)
        self.delBtn.clicked.connect(self.delEntries)
        self.dirBtn.clicked.connect(self.outputDir)
        self.submit.clicked.connect(self.rip)

        self.submitMovie.frameChanged.connect(self.setSubmitIcon)
        
        self.submitMovie.start()
        self.submitMovie.stop()

        #setup file chooser fileDialog
        self.fileDialog = QtWidgets.QFileDialog(self)


        self.browser = Dialog()
        self.browser.setParent(self)

        self.help = HDialog()
        self.help.setHtml()

        self.item = None

        #debug
        home= os.path.expanduser("~")
        desktop= os.path.join(home,"Desktop")

        self.outputPath.setText(desktop)
        #end debug
        
        self.liveThreads=0;


    def setSubmitIcon(self):
        saw_icon = QtGui.QIcon()
        saw_icon.addPixmap(self.submitMovie.currentPixmap(), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.submit.setIcon(saw_icon)



    # Add an entry to the list...
    def addEntry(self):
        _time = str(time.time())

        # custom list item...
        ui_item =   ListItemGeneric()
        ui_item.setParent(self)


        # set the list widget item to the list...
        li_item = QtWidgets.QListWidgetItem(self.list)

        # set the size hint...
        li_item.setSizeHint(ui_item.sizeHint())
 
        self.list.addItem(li_item)
        self.list.setItemWidget(li_item,ui_item)


    # Delete entries from the list..
    def delEntries(self):
        item = self.list.takeItem(self.list.currentRow())
        item = None

    
    def browserShow(self):
        self.browser.show()

    def webUrlSelected(self,text):
        self.item.setWebUrl(text)

    def outputDir(self):
        self.outputPath.setText(self.fileDialog.getExistingDirectory())


    def rip(self):
        if (self.liveThreads > 0):
            QtWidgets.QMessageBox.about(self,"ERROR","Already running homie.")
            return 

        if(self.outputPath.text() == ""):
            QtWidgets.QMessageBox.about(self,"ERROR","Asere you forgot the output path.\nDonde quieres los files?")
        else:
            _start = False
            self.threads = []
            for item in self.list.findItems('',QtCore.Qt.MatchRegExp):

                # #set up the item 
                _item = self.list.itemWidget(item)
                _item.setOutputPath(self.outputPath.text())
                _item.setUIMain(self)
                _item.rip()
                self.liveThreads=self.liveThreads+1
                time.sleep(2)
                _start = True

            if _start:
                self.submitMovie.start()
 

    def progressReady(self,li,_max):
        li.progress.show()
        li.progress.setRange(1,_max)
        QtWidgets.qApp.processEvents()
        

    def doProgress(self,li,_value,_message):
        li.progress.setValue(_value)
        li.outputLabel.setText(_message)
        QtWidgets.qApp.processEvents()

    def finished(self):
        self.liveThreads-=1

        if self.liveThreads <= 0 :
             self.submitMovie.stop()

    def updateLibs(self):
        # add headers for cookie redirects.
        self.shell=os.path.join(os.path.expanduser("~"),".mediaripper","shell");
        self.msg = QtWidgets.QMessageBox()
        self.msg.setIcon(QtWidgets.QMessageBox.Information)

        # set the root folder..
        home = os.path.expanduser("~")
        mediaripper = os.path.join(home,".mediaripper")

        # set the different assets used.
        youtube_dl = os.path.join(mediaripper,"youtube_dl")

        self.msg.setText("Updating your Libraries")
        self.msg.setWindowTitle("ASERE!")
        self.msg.setInformativeText("Refreshing...")
        self.msg.setDetailedText("You can view these libraries at:\n ~/.mediaripper/\n ")   
            
        self.msg.setStandardButtons(QtWidgets.QMessageBox.Ok|QtWidgets.QMessageBox.Cancel)    
        retval = self.msg.exec_()
 
        if bool((retval & QtWidgets.QMessageBox.Ok)):

            try:

                p= sub.Popen([self.shell,"-u" ],shell=False,stdout=sub.PIPE, stderr=sub.PIPE)
                pipe = p.stdout
                err =  p.stderr

                while  True:
                    b = pipe.readline()

                    if not b: 
                        pipe.close()
                        err.close()
                        break


            except Exception as e:
               err = 'Error on line {}' + format(sys.exc_info()[-1].tb_lineno)+ type(e).__name__+ str(e);
               print(str(err))

            self.msg.setInformativeText("Libraries Refreshed have fun!")
            self.msg.exec_()
        else:
            return

    def postShow(self):
        self.checkResourceExistance()

    def checkResourceExistance(self):

        lib= Strings.libraryPath()
        home = os.path.expanduser("~")
        mediaripper = os.path.join(home,".mediaripper")

        youtube_dl = False
        ffmpeg = False
        shell=False;
    

        if not os.path.exists(mediaripper):
            os.makedirs(mediaripper)
            os.chmod(mediaripper,0o755)

        for filename in os.listdir(mediaripper):
            if filename == "ffmpeg":
                ffmpeg = True
            if filename == "youtube_dl":
                youtube_dl = True
            if filename == "shell":
                shell = True
     

        if not shell:
            shutil.copy(os.path.join(lib,".mediaripper","shell"),os.path.join(mediaripper,"shell"));
        if not youtube_dl:
            shutil.copy(os.path.join(lib,".mediaripper","youtube_dl"),os.path.join(mediaripper,"youtube_dl"));
        if not ffmpeg:
            shutil.copy(os.path.join(lib,".mediaripper","ffmpeg"),os.path.join(mediaripper,"ffmpeg"));

    def showHelp(self):
          self.help.show()



# Main Method
# opens the gui and starts the application..
if __name__ == "__main__":
    app = QtWidgets.QApplication(sys.argv)
    window = MyApp()
    window.show()
    time.sleep(1)
    window.postShow()
    sys.exit(app.exec_())